"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// ../packages/core/index.ts
var core_exports = {};
__export(core_exports, {
  run: () => run
});
module.exports = __toCommonJS(core_exports);
var process2 = __toESM(require("process"), 1);
var import_utils2 = require("../utils/index.cjs");
var import_cac = require("cac");

// ../package.json
var version = "1.0.0";

// ../packages/core/storage.ts
var import_fs = require("fs");
var import_path = require("path");
var import_os = require("os");
var storage;
var storageDir = (0, import_path.resolve)((0, import_os.tmpdir)(), "pr-storage");
var storagePath = (0, import_path.resolve)(storageDir, "_pr_storage.json");
async function loadStorage(fn) {
  if (!storage) {
    storage = (0, import_fs.existsSync)(storagePath) ? JSON.parse(await import_fs.promises.readFile(storagePath, "utf-8") || "{}") || {} : {};
  }
  if (fn) {
    if (await fn(storage))
      await saveStorage();
  }
  return storage;
}
async function saveStorage() {
  if (storage) {
    if (!(0, import_fs.existsSync)(storageDir))
      await import_fs.promises.mkdir(storageDir, { recursive: true });
    await import_fs.promises.writeFile(storagePath, JSON.stringify(storage), "utf-8");
  }
}
async function clearStorage() {
  if (storage) {
    if (!(0, import_fs.existsSync)(storageDir))
      await import_fs.promises.mkdir(storageDir, { recursive: true });
    await import_fs.promises.writeFile(storagePath, "{}", "utf-8");
  }
}

// ../packages/core/runtime.ts
var import_prompts = __toESM(require("prompts"), 1);
var import_ora = __toESM(require("ora"), 1);
var import_utils = require("../utils/index.cjs");
var import_console_table_printer = require("console-table-printer");
var import_chalk2 = __toESM(require("chalk"), 1);

// ../packages/core/command-option.ts
var import_chalk = __toESM(require("chalk"), 1);
var typeOption = [{
  type: "select",
  name: "typeSelect",
  message: "Detect all Repo's PR?",
  choices: ["All Repo", "Detect the PR of a certain Repo"].map((item) => {
    let value = "all";
    if (item !== "All Repo")
      value = "select";
    return { title: item, value };
  })
}];
var createRepoOption = (list) => {
  const handler = (item) => {
    return { title: item, value: item };
  };
  return [{
    type: "select",
    name: "RepoSelect",
    message: "Please select a Repo",
    choices: list.map(handler)
  }];
};
var createPrOption = (list) => {
  const handler = (item) => {
    const repo = import_chalk.default.blueBright.bold(`[${item.repo}]`);
    const number = import_chalk.default.yellowBright.bold(`[#${item.number}]`);
    const title = `${item.title}`;
    const canMerge = item.isNeedUpdate ? import_chalk.default.yellowBright.bold("<CanMerge:true>") : import_chalk.default.redBright.bold("<CanMerge:false>");
    const infoTitle = `${canMerge}: ${repo}-${number} -> ${title}`;
    return {
      title: infoTitle,
      value: {
        title: item.title,
        number: item.number,
        repo: item.repo,
        isNeedUpdate: item.isNeedUpdate,
        infoTitle
      }
    };
  };
  return [{
    type: "multiselect",
    name: "prSelect",
    message: "Please select the PR that needs to be updated",
    choices: list.map(handler)
  }];
};

// ../packages/core/gitApi.ts
var import_core = require("@octokit/core");
var GitApi = class {
  octokit;
  owner;
  constructor(token, owner) {
    this.octokit = new import_core.Octokit({
      auth: token
    });
    this.owner = owner;
  }
  /**
   * 获取账户下所有 pr
   * @param username
   */
  async getPRList(username = this.owner) {
    try {
      const { data } = await this.octokit.request("GET /search/issues", {
        q: `is:pr is:open author:${username}`,
        per_page: 1e3
      });
      const res = {};
      data.items.forEach((val) => {
        const repo = val.repository_url.split("repos/")[1];
        if (!res[repo])
          res[repo] = [];
        res[repo].push({
          title: val.title,
          number: val.number,
          repo,
          id: val.id
        });
      });
      return res;
    } catch (error) {
      console.log(error);
      return [];
    }
  }
  /**
   * 根据  pr number 与 repo name 获取 pr 详情
   * @param pull_number pr 号
   * @param repo_name 源仓库名
   */
  async getPRByRepo(pull_number, repo_name, title) {
    try {
      const { data } = await this.octokit.request(
        `GET /repos/${repo_name}/pulls/{pull_number}`,
        {
          pull_number
        }
      );
      return {
        sha: data.base.sha,
        org_repo: data.base.repo.full_name,
        org_repo_default_branch: data.base.repo.default_branch,
        pr_sha: data.head.sha,
        pr_repo: data.head.repo.full_name,
        pr_repo_default_branch: data.head.repo.default_branch,
        merged: data.merged,
        mergeable: data.mergeable,
        number: pull_number,
        repo: repo_name,
        title: title || ""
      };
    } catch (error) {
      console.error(error);
      return {};
    }
  }
  /**
   * 判断 pr 是否要同步源仓库
   * @param repo_name 源仓库名
   * @param pr_info 该 pr 版本中， fork 的仓库提交 hash
   */
  async needUpdate(repo_name, pr_info) {
    try {
      const { data } = await this.octokit.request(
        `GET /repos/${repo_name}/commits`
      );
      if (data.length >= 0) {
        const cur_sha = data[0].sha;
        if (pr_info.sha !== cur_sha && pr_info.mergeable && !pr_info.merged)
          return { isNeedUpdate: true };
        else
          return { isNeedUpdate: false };
      } else {
        return { isNeedUpdate: false };
      }
    } catch (error) {
      console.error(error);
      return { isNeedUpdate: false };
    }
  }
  /**
   * 更新pr
   * @param pull_number
   * @param repo_name
   */
  async updatePR(pull_number, repo_name) {
    try {
      const res = await this.octokit.request(
        `PUT /repos/${repo_name}/pulls/{pull_number}/update-branch`,
        {
          pull_number
        }
      );
      return res.data.message;
    } catch (error) {
      console.error(error);
      return {};
    }
  }
};

// ../packages/core/runtime.ts
var promptsRun = async (option) => {
  const res = await (0, import_prompts.default)(option);
  return { ...res };
};
async function runtimeStart(store) {
  const isAllRepo = await promptsRun(typeOption);
  let updateRes = [];
  let spinner = (0, import_ora.default)({ text: "Loading Repo......", color: "blue" }).start();
  const githubApi = new GitApi(store.token, store.username);
  const prList = await githubApi.getPRList();
  spinner.stop();
  const repoList = Object.keys(prList);
  if (isAllRepo.typeSelect === "select") {
    const selectRepo = await promptsRun(createRepoOption(repoList));
    const prl = prList[selectRepo.RepoSelect];
    spinner = (0, import_ora.default)({ text: `Checking PR by ${selectRepo.RepoSelect}......`, color: "blue" }).start();
    const prListByRepo = await checkPR(prl, githubApi);
    spinner.stop();
    const prSelectRes = await promptsRun(createPrOption(prListByRepo));
    spinner = (0, import_ora.default)({ text: `Update PR by ${selectRepo.RepoSelect}......`, color: "blue" }).start();
    updateRes = await updatePR(prl, prSelectRes, githubApi);
  } else {
    const prl = [];
    repoList.forEach((val) => {
      prList[val].forEach((item) => {
        prl.push(item);
      });
    });
    spinner = (0, import_ora.default)({ text: "Checking PR......", color: "blue" }).start();
    const prListByRepo = await checkPR(prl, githubApi);
    spinner.stop();
    const prSelectRes = await promptsRun(createPrOption(prListByRepo));
    spinner = (0, import_ora.default)({ text: "Update PR......", color: "blue" }).start();
    updateRes = await updatePR(prl, prSelectRes, githubApi);
  }
  spinner.succeed();
  (0, import_utils.log)("success", "\u2714 All PR updates completed");
  await printUpdateRes(updateRes);
}
async function checkPR(prl, githubApi) {
  if (prl.length === 0) {
    (0, import_utils.log)("error", `Please select a pr to check`);
    process.exit();
  }
  const prListByRepo = [];
  for (let i = 0; i < prl.length; i++) {
    const prInfo = await githubApi.getPRByRepo(prl[i].number, prl[i].repo, prl[i].title);
    const res = await githubApi.needUpdate(prl[i].repo, prInfo);
    (0, import_utils.log)("success", `\u2714 NO.${i + 1}:Check PR #${prl[i].number} completed`);
    prListByRepo.push({
      ...prInfo,
      isNeedUpdate: res.isNeedUpdate
    });
  }
  return prListByRepo;
}
async function updatePR(prl, prSelectRes, githubApi) {
  const updateRes = [];
  for (let i = 0; i < prSelectRes.prSelect.length; i++) {
    if (prSelectRes.prSelect[i].isNeedUpdate) {
      (0, import_utils.log)("success", `\u2714 NO.${i + 1}:update PR #${prl[i].number} completed`);
    }
    updateRes.push({
      ...prSelectRes.prSelect[i]
    });
  }
  return updateRes;
}
async function printUpdateRes(res) {
  const p = new import_console_table_printer.Table({
    columns: [
      { name: "number", alignment: "left" },
      { name: "can merge", alignment: "left" },
      { name: "success", alignment: "left" },
      { name: "repo", alignment: "left" },
      { name: "title", alignment: "left" }
    ]
  });
  res.sort((a, b) => Number(a.isNeedUpdate) - Number(b.isNeedUpdate));
  res.forEach((item) => {
    p.addRow({
      "number": import_chalk2.default.greenBright.bold(`#${item.number}`),
      "can merge": item.isNeedUpdate ? import_chalk2.default.greenBright.bold("true") : import_chalk2.default.redBright.bold("false"),
      "success": item.isNeedUpdate ? import_chalk2.default.greenBright.bold("true") : import_chalk2.default.redBright.bold("false"),
      "repo": import_chalk2.default.blueBright.bold(`<${item.repo}>`),
      "title": item.title
    });
  });
  p.printTable();
}

// ../packages/core/index.ts
var cli = (0, import_cac.cac)("pr-checker");
var run = async () => {
  const storage2 = await loadStorage();
  cli.option("-t <token>,--token <token>", "set github token");
  cli.option("-u <username>, --username <username>", "set github username");
  cli.option("-c, --clear", "clear token and username");
  cli.option("-g, --get", "get git config");
  cli.command("run", "check your pr").action(async () => {
    if (!storage2.token || !storage2.username) {
      (0, import_utils2.log)("error", "use `pr-checker -t <TOKEN>` to set your token");
      process2.exit(1);
      return;
    }
    await runtimeStart(storage2);
  });
  cli.help();
  cli.version(version);
  const parseRes = cli.parse();
  const {
    u,
    username,
    t,
    token,
    c,
    clear,
    h,
    help,
    v,
    version: versions,
    g,
    get
  } = parseRes.options;
  if (h || help || v || versions)
    return;
  if (c || clear) {
    if (storage2) {
      await clearStorage();
      (0, import_utils2.log)("success", "clear token and username successfully");
      return;
    }
  }
  if (g || get) {
    if (storage2) {
      (0, import_utils2.log)("info", `username: ${storage2.username || "null"}`);
      (0, import_utils2.log)("info", `token: ${storage2.token || "null"}`);
      return;
    }
  }
  if (t && typeof t === "string" || token && typeof token === "string") {
    storage2.token = t || token;
    await saveStorage();
    (0, import_utils2.log)("success", "token set successfully");
    return;
  }
  if (u && typeof u === "string" || username && typeof username === "string") {
    storage2.username = u || username;
    await saveStorage();
    (0, import_utils2.log)("success", "username set successfully");
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  run
});
